//
// Chiptune Mega Mega
// Yunji Fengshi (yf88) & Scott Zhao (sz296)
// 

//
// Includes, defines, globals
//
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <inttypes.h>
#include <avr/pgmspace.h>

#define F_CPU 16000000UL // 16 MHz clock

#include <util/delay.h>
#include <stdio.h>
#include <string.h>
#include "lcd_lib.h"

// NULL indicates the end of a track
#define NULL 0

// NULL_SAMP indicates the end of a sample
#define NULL_SAMP 0x67

// LCD strings
const char lcd_juke[] PROGMEM = "Jukebox Mode   ";
const char lcd_affective[] PROGMEM = "Affective Mode ";
const char lcd_bruises[] PROGMEM = "*Bruises";
const char lcd_forever[] PROGMEM = "*Faye";
const char lcd_phoenix[] PROGMEM = "*Phoenix";

// User interface
volatile uint8_t u_pressed = 0;
volatile uint8_t u_debounce_time = 0;
volatile uint8_t d_pressed = 0;
volatile uint8_t d_debounce_time = 0;
volatile uint8_t s_pressed = 0;
volatile uint8_t s_debounce_time = 0;

uint8_t juke_go = 0;
uint8_t change_mode = 0;

#define MODE 0
#define BRUISES 1
#define FOREVER 2
#define PHOENIX 3
uint8_t cursor_pos = MODE;

#define JUKE 0
#define AFFECTIVE 1
uint8_t mode = JUKE;

// Special commands
#define R 1
#define Sw 2
#define Du0 3
#define Du1 4
#define Du2 5
#define Du3 6
#define Vol 7
#define COMMAND 7

// Frequencies of musical notes
#define C0 16
#define Db0 17
#define D0 18
#define Eb0 19
#define E0 21
#define F0 22
#define Gb0 23
#define G0 25
#define Ab0 26
#define A0 28
#define Bb0 29
#define B0 31
#define C1 33
#define Db1 35
#define D1 37
#define Eb1 39
#define E1 41
#define F1 44
#define Gb1 46
#define G1 49
#define Ab1 52
#define A1 55
#define Bb1 58
#define B1 62
#define C2 65
#define Db2 69
#define D2 73
#define Eb2 78
#define E2 82
#define F2 87
#define Gb2 93
#define G2 98
#define Ab2 104
#define A2 110
#define Bb2 117
#define B2 123
#define C3 131
#define Db3 139
#define D3 147
#define Eb3 156
#define E3 165
#define F3 175
#define Gb3 185
#define G3 196
#define Ab3 208
#define A3 220
#define Bb3 233
#define B3 247
#define C4 261
#define Db4 277
#define D4 294
#define Eb4 311
#define E4 330
#define F4 349
#define Gb4 370
#define G4 392
#define Ab4 415
#define A4 440
#define Bb4 466
#define B4 494
#define C5 523
#define Db5 554
#define D5 587
#define Eb5 622
#define E5 659
#define F5 698
#define Gb5 740
#define G5 783
#define Ab5 831

// Sharps
#define Fs1 Gb1
#define Cs2 Db2
#define Ds2 Eb2
#define Fs2 Gb2
#define Gs2 Ab2
#define Cs3 Db3
#define Ds3 Eb3
#define Fs3 Gb3
#define Gs3 Ab3
#define Cs4 Db4
#define Ds4 Eb4
#define Fs4 Gb4
#define Gs4 Ab4
#define Cs5 Db5
#define Ds5 Eb5

// Noise channel presets
#define H 2
/*
#define N0 220
#define N1 440
#define N2 880
#define N3 1175
#define N4 1762
#define N5 2355
#define N6 3524
#define N7 4430
#define N8 5594
#define N9 6992
#define N10 9323
#define N11 13984
#define N12 27969
#define N13 27969
#define N14 27969
#define N15 27969
*/

// DPCM pitch table
#define Kick 2
#define Rim 3
/*
#define DMC0 4182
#define DMC1 4710
#define DMC2 5264
#define DMC3 5593
#define DMC4 6258
#define DMC5 7046
#define DMC6 7919
#define DMC7 8363
#define DMC8 9420
#define DMC9 11186
#define DMC10 12604
#define DMC11 13983
#define DMC12 16885
#define DMC13 21307
#define DMC14 24858
#define DMC15 33144
*/

// Sweep unit
#define SHIFT_COUNT 7

// Hex values for transcribing rhythm
#define _1_1 0x0101
#define _1_2 0x0102
#define _1_4 0x0104
#define _3_4 0x0304
#define _1_8 0x0108
#define _3_8 0x0308
#define _5_8 0x0508
#define _7_8 0x0708
#define _1_16 0x0110
#define _3_16 0x0310
#define _5_16 0x0510
#define _7_16 0x0710
#define _9_16 0x0910
#define _1_32 0x0120
#define _3_32 0x0320
#define _7_32 0x0720
#define _19_32 0x1320
#define _1_64 0x0140
#define _3_64 0x0340
#define _7_64 0x0740
#define _39_64 0x2740

// Coefficient for scaling musical frequencies into an increment
#define INC_SCALAR 2.097152

// Tempo
uint8_t bpm_current = 60;
uint8_t bpm_desired = 60;
uint16_t beat_dur = 240000 / 60;
uint8_t bpm_go_up = 1;

// Timing
#define TICKS_PER_MS 62
volatile uint8_t base_time = TICKS_PER_MS;

volatile uint16_t p1_accumulator = 0; // increment through lower three bytes for additional resolution
volatile uint8_t p1_sequencer = 0; // highest three bits of accumulator for sequencing waveform
volatile uint16_t p1_increment = 0; // step size for traversing waveform table; function of note frequency

volatile uint16_t p1_length = 0;
volatile uint8_t p1_silence = 1;
volatile uint8_t p1_ready = 1;

volatile uint8_t s_go = 0;
uint16_t s_des_increment = 0;
volatile uint16_t s_increment = 0;
volatile uint8_t s_up = 0;
volatile uint8_t s_divider = 240;

volatile uint16_t t_accumulator = 0;
volatile uint8_t t_sequencer = 0;
volatile uint16_t t_increment = 0;

volatile uint16_t t_length = 0;
volatile uint8_t t_silence = 1;
volatile uint8_t t_ready = 1;

volatile uint8_t b_go = 0;
uint16_t b_des_increment = 0;
volatile uint16_t b_increment = 0;
volatile uint8_t b_up = 0;
volatile uint8_t b_divider = 240;

volatile uint16_t n_accumulator = 0;
volatile uint16_t n_increment = 0;
volatile uint8_t n_overflowed = 0;
volatile uint16_t n_last_accumulator = 0;

volatile uint16_t n_length = 0;
volatile uint8_t n_silence = 1;
volatile uint8_t n_ready = 1;

const uint8_t mario_bass_kick[] = { 0x55, 0x55, 0x55, 0x95, 0xAA, 0x2A, 0x95, 0xE0, 0x7F, 0xFC, 0xC0, 0xF1, 0x03, 0x28, 0xFE, 0xFF, 0xFF, 0xF1, 0x5F, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x80, 0xC0, 0xF1, 0xFF, 0xC7, 0x8B, 0x1F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xAB, 0x6A, 0x9B, 0xAA, 0x92, 0x52, 0x25, 0x95, 0x24, 0x25, 0x92, 0x80, 0x4A, 0x50, 0x05, 0x00, 0x00, 0x80, 0x20, 0x42, 0x04, 0x08, 0xE2, 0xFF, 0x01, 0x80, 0x6D, 0xAB, 0x6D, 0xDF, 0xBB, 0x77, 0x7B, 0xAB, 0xAF, 0xDD, 0xD5, 0x7D, 0xDD, 0xBF, 0xFF, 0xEF, 0xB6, 0x6D, 0x6F, 0xBB, 0x6D, 0xAF, 0x2A, 0x95, 0x94, 0x24, 0x49, 0x92, 0x88, 0x42, 0x84, 0x48, 0x88, 0x04, 0x09, 0x49, 0x92, 0x24, 0x51, 0x22, 0xA5, 0x92, 0x22, 0x49, 0xAA, 0x52, 0xA9, 0xAA, 0x5A, 0x55, 0xAD, 0x55, 0x55, 0xAD, 0x6D, 0xB5, 0xAD, 0x6D, 0xB5, 0x6D, 0xDB, 0x55, NULL_SAMP };
const uint8_t tecmo_bass_kick[] = {
	0x9A, 0x71, 0x96, 0x99, 0xA3, 0x69, 0x96, 0x99, 0xF9, 0x88, 0x9D, 0x9B, 0x99, 0xCD, 0x66, 0xDA,
	0xCA, 0xB0, 0xB3, 0x44, 0xCB, 0x71, 0x1C, 0x83, 0x3C, 0xF8, 0x07, 0x00, 0x01, 0xF8, 0x07, 0xC0,
	0x1F, 0xC0, 0x03, 0x42, 0xF6, 0x87, 0x18, 0x5F, 0xF3, 0xA5, 0xEB, 0x3F, 0x93, 0xDF, 0xBF, 0xE4,
	0xBD, 0xF9, 0x7D, 0x8B, 0xB7, 0x37, 0xF3, 0x7C, 0x9C, 0xF9, 0x1C, 0xA5, 0x13, 0x97, 0x66, 0x99,
	0x6A, 0x54, 0x62, 0x44, 0x49, 0xC1, 0x12, 0x91, 0x78, 0x04, 0x6A, 0x32, 0x02, 0x33, 0x0B, 0x46,
	0x1A, 0x93, 0x91, 0x61, 0x5A, 0x14, 0x5B, 0x26, 0x99, 0xCC, 0x64, 0xE5, 0x16, 0x95, 0xAB, 0x95,
	0xE3, 0x1C, 0x6B, 0x99, 0xCB, 0xCB, 0x99, 0xE6, 0x69, 0xD6, 0x6C, 0x36, 0x9B, 0xCB, 0xA9, 0x39,
	0xDB, 0xB8, 0x72, 0xB9, 0x5C, 0x4E, 0x2D, 0xA7, 0x93, 0xE5, 0xA9, 0xE1, 0xCC, 0x65, 0xB1, 0x5A,
	0x55, 0x39, 0xA9, 0xB5, 0x52, 0xAB, 0x52, 0x55, 0x99, 0x59, 0x56, 0xE5, 0x64, 0x5A, 0x59, 0x59,
	0x66, 0x56, 0x55, 0x55, 0x55, 0x99, 0x55, 0x8E, 0x65, 0x69, 0x99, 0xB2, 0x9C, 0x2C, 0x47, 0x93,
	0x33, 0xD5, 0x54, 0x55, 0x59, 0xA6, 0xA6, 0xCA, 0xC9, 0xD4, 0x4C, 0x56, 0x99, 0xE5, 0xA8, 0x9A,
	0xCA, 0xAA, 0x26, 0x53, 0xD3, 0x58, 0xA5, 0xA9, 0x2C, 0x4D, 0x65, 0xA9, 0x32, 0x95, 0xC5, 0xCC,
	0x94, 0xA9, 0x2A, 0x53, 0x95, 0x69, 0x2A, 0xCB, 0x32, 0x53, 0x55, 0xD5, 0x38, 0x35, 0xD5, 0xD2,
	0xAC, 0xAA, 0x65, 0x96, 0xAB, 0xD4, 0xAC, 0x95, 0x59, 0x35, 0xAB, 0xAA, 0x56, 0x55, 0xB5, 0xAA,
	0xAA, 0x5A, 0x55, 0x35, 0xAB, 0xAA, 0x6A, 0x56, 0x55, 0x55, 0xB5, 0xAA, 0xAA, 0x9A, 0xA9, 0x65,
	0x59, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x96, 0x65, 0x66, 0x6A, 0xAA, 0xAA,
	0xAA,
	NULL_SAMP
};
const uint8_t tecmo_rim_shot[] =
{
	0x15, 0xF5, 0x3F, 0x40, 0xE1, 0x1F, 0x00, 0xFC, 0x7F, 0x00, 0xFF, 0x1F, 0x00, 0xC0, 0xFF, 0xE3,
	0xB0, 0xCC, 0x0E, 0x81, 0x1F, 0x03, 0xC0, 0x3F, 0xFF, 0x7F, 0x00, 0xF8, 0x1F, 0x00, 0xFE, 0x1F,
	0x00, 0x7F, 0x78, 0x7E, 0x00, 0xF0, 0xFF, 0x03, 0x30, 0xF8, 0x1C, 0xC0, 0xFF, 0xF3, 0x40, 0x7C,
	0xD0, 0x57, 0x01, 0xF4, 0x5E, 0x18, 0xFC, 0x71, 0x3F, 0x94, 0x0A, 0xF0, 0x03, 0x3F, 0x86, 0x8F,
	0x5F, 0x0C, 0x98, 0xFF, 0x07, 0xDC, 0x0F, 0x00, 0xF0, 0xFF, 0x03, 0xF8, 0x01, 0xFE, 0x07, 0x0F,
	0x1E, 0x00, 0xFF, 0xFF, 0x01, 0xF0, 0x03, 0xFC, 0x01, 0xFE, 0xC0, 0x71, 0xFE, 0x1F, 0x18, 0x00,
	0xFF, 0x01, 0xFE, 0x01, 0xFE, 0x07, 0x00, 0xFE, 0x01, 0xFE, 0xFF, 0x01, 0xF8, 0x07, 0xF0, 0x5F,
	0x00, 0xFC, 0x7F, 0x00, 0x6D, 0x07, 0xC0, 0xFF, 0x3F, 0x00, 0xFE, 0x0F, 0x40, 0xC0, 0xFF, 0x03,
	0xC0, 0xFF, 0x1F, 0x00, 0xFE, 0xE5, 0x50, 0x1F, 0xC0, 0xBF, 0x00, 0xFC, 0x27, 0x7F, 0xC0, 0x00,
	0xF8, 0xFF, 0x7D, 0x00, 0x0E, 0xE0, 0x7F, 0xC0, 0xFF, 0x05, 0x00, 0xFF, 0x0F, 0xC0, 0x7F, 0x00,
	0xE8, 0xFF, 0x03, 0xE8, 0x5C, 0x7D, 0x0D, 0xE0, 0x27, 0x01, 0xFF, 0x5F, 0x08, 0x40, 0xF1, 0xFF,
	0x81, 0xDE, 0x1A, 0x00, 0xFF, 0x01, 0xF8, 0xFF, 0x41, 0x80, 0xDA, 0xFE, 0x01, 0xF0, 0x7F, 0x80,
	0x2F, 0xC0, 0xFF, 0x07, 0xC0, 0x7F, 0xA0, 0xE0, 0x7F, 0x00, 0xFE, 0x03, 0x05, 0x6F, 0xF5, 0x0F,
	0xC0, 0x2F, 0x7E, 0x08, 0x7D, 0x00, 0xFF, 0x07, 0xF0, 0x07, 0xF0, 0x9F, 0x40, 0x7F, 0x80, 0x5F,
	0xFF, 0x01, 0x50, 0xF2, 0x57, 0x06, 0xC4, 0x7F, 0xE0, 0x07, 0xFC, 0x81, 0x6B, 0x2D, 0xF8, 0x2F,
	0x24, 0x60, 0xFF, 0x00, 0x9F, 0xF4, 0x27, 0x80, 0xFF, 0x80, 0xEC, 0x9F, 0x40, 0xD5, 0xD4, 0xAF,
	0x02, 0xF8, 0xBC, 0x5A, 0xA0, 0x5E, 0xD2, 0x17, 0xE8, 0x0F, 0xFC, 0x09, 0xF0, 0x97, 0x40, 0x3F,
	0xA1, 0x3F, 0x44, 0xEA, 0x0F, 0x78, 0xD5, 0xA8, 0xE8, 0x0F, 0xF0, 0x4F, 0x34, 0xD0, 0x5F, 0x28,
	0x5F, 0x52, 0x25, 0x8B, 0x4F, 0x0B, 0xD2, 0xFF, 0x00, 0x75, 0x6D, 0x45, 0xB2, 0x92, 0xDE, 0x49,
	0x5D, 0x04, 0xBB, 0x2B, 0xC9, 0x0F, 0x29, 0xE9, 0xBD, 0x20, 0xAB, 0xD4, 0x57, 0x48, 0xF6, 0x89,
	0xAA, 0xE8, 0x57, 0x52, 0x52, 0xAB, 0x55, 0x2B, 0x49, 0xBD, 0x54, 0xB2, 0xB5, 0x22, 0x6B, 0x47,
	0xB4, 0xA6, 0xEC, 0x9A, 0xA0, 0x5B, 0x55, 0x95, 0xD4, 0xBA, 0x52, 0x6A, 0xAA, 0x4E, 0x6A, 0x17,
	0xD2, 0x66, 0x55, 0xAA, 0x15, 0xEB, 0x4A, 0x5A, 0xA9, 0x55, 0x74, 0x4B, 0x74, 0x2B, 0xAA, 0x55,
	0x13, 0xB9, 0x27, 0xA9, 0xAA, 0x4D, 0xD2, 0x2B, 0x69, 0xA5, 0xB6, 0x44, 0x57, 0x55, 0x56, 0xAA,
	0x56, 0xC9, 0x56, 0xA9, 0x55, 0x55, 0x55, 0xAA, 0x6A, 0x55, 0x55, 0x55, 0xA9, 0x66, 0x95, 0x5A,
	0x95, 0x6A, 0x55, 0x95, 0xAA, 0x5A, 0x2B, 0xD1, 0x56, 0xA9, 0xAA, 0xAD, 0xA4, 0x5A, 0x95, 0x5A,
	0x95, 0x5A, 0x95, 0x6A, 0x55, 0x55, 0xA9, 0xB6, 0xA4, 0x5A, 0xA9, 0x5A, 0x55, 0xAA, 0x56, 0x55,
	0xD2, 0x66, 0x55, 0xAA, 0x15, 0xEB, 0x4A, 0x5A, 0xA9, 0x55, 0x74, 0x4B, 0x74, 0x2B, 0xAA, 0x55,
	0x13, 0xB9, 0x27, 0xA9, 0xAA, 0x4D, 0xD2, 0x2B, 0x69, 0xA5, 0xB6, 0x44, 0x57, 0x55, 0x56, 0xAA,
	0x56, 0xC9, 0x56, 0xA9, 0x55, 0x55, 0x55, 0xAA, 0x6A, 0x55, 0x55, 0x55, 0xA9, 0x66, 0x95, 0x5A,
	0x95, 0x6A, 0x55, 0x95, 0xAA, 0x5A, 0x2B, 0xD1, 0x56, 0xA9, 0xAA, 0xAD, 0xA4, 0x5A, 0x95, 0x5A,
	0x95,
	NULL_SAMP
};

volatile const uint8_t* d_sample = 0;
volatile uint8_t d_bits_rem = 8;
volatile uint8_t d_register = 0;
volatile uint8_t d_output = 64;
volatile uint8_t d_last_output = 0;

volatile uint16_t d_length = 0;
volatile uint8_t d_ready = 1;
volatile uint8_t d_silence = 1;

volatile uint8_t pulse_go = 1;

// Pulse waveform sequences
const uint8_t p1_duty0[8] = { 0, 0, 0, 0, 0, 0, 0, 1 }; // 12.5% duty cycle
const uint8_t p1_duty1[8] = { 0, 0, 0, 0, 0, 0, 1, 1 }; // 25% duty cycle
const uint8_t p1_duty2[8] = { 0, 0, 0, 0, 1, 1, 1, 1 }; // 50% duty cycle
const uint8_t p1_duty3[8] = { 0, 0, 1, 1, 1, 1, 1, 1 }; // 75% duty cycle (25% negated)
volatile uint16_t p1_duty = 2;

// Pulse channel gain setting; for debugging
#define P1_ATTENUATION 4
#define P1_OUTPUT_LEVEL (256 / P1_ATTENUATION + 1)

// Pulse channel ramp
/*
uint8_t ramp[256] = {0};
volatile uint8_t p1_sample = 0;
volatile uint8_t r_counter = 0;
#define RAMP_DUR 4 // duration of linear ramp in milliseconds
#define RAMP_UP_END 125
*/

// 4-bit triangle waveform sequence
const uint8_t triangle[32] = {	15, 14, 13, 12, 11, 10, 9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
								0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 };
/*
const uint8_t triangle[64] =
{
	31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0,
	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
};
*/
/*
const uint8_t triangle[128] =
{
	63, 62, 61, 60, 59, 58, 57, 56, 55, 54, 53, 52, 51, 50, 49, 48, 47, 46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32,
	31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0,
	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
	32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63
};
*/
								
// Noise shift register
volatile uint16_t noise = 1;
								
// Tracks
const uint16_t bruises_pulse1[] =
{
	R, _1_1,																							// Intro
	R, _1_1,																							// Measure 1
	R, _1_1,																							// Measure 2
	R, _1_1,																							// Measure 3
	R, _1_1,																							// Measure 4
	R, _1_1,																							// Measure 5
	R, _1_1,																							// Measure 6
	R, _1_1,																							// Measure 7
	R, _7_8, G3, _1_8,																					// Measure 8
	C4, _1_8, A3, _1_16, C4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _1_8, A3, _3_16,					// Measure 9
	C4, _1_8, A3, _1_16, C4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _3_16, R, _1_8,					// Measure 10
	C4, _1_8, A3, _1_16, C4, _1_8, D4, _3_16, E4, _1_16, D4, _1_8, C4, _1_8, G3, _3_16,					// Measure 11
	C4, _1_8, G3, _1_16, C4, _1_8, D4, _3_16, E4, _1_8, R, _3_16, Sw, E4, G4, _3_16,					// Measure 12
	G4, _1_8, E4, _1_16, E4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _3_16, G4, _1_8,					// Measure 13
	G4, _1_8, A4, _1_8, E4, _1_8, D4, _1_8, E4, _1_8, D4, _1_16, C4, _3_16, A3, _1_8,					// Measure 14
	G4, _1_8, E4, _1_16, E4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _3_16, R, _1_8,					// Measure 15
	G4, _1_8, E4, _1_16, E4, _1_8, D4, _3_16, Sw, D4, E4, _1_4, B3, _1_4,								// Measure 16
	Du2, G4, _1_64, Du1, G4, _39_64, E4, _1_8, G4, _1_8, E4, _1_8,										// Measure 17
	D4, _1_8, E4, _1_16, A3, _7_16, C4, _1_8, G4, _1_8, E4, _1_8,										// Measure 18
	D4, _1_8, E4, _1_16, A3, _7_16, C4, _1_8, G4, _1_8, C4, _1_8,										// Measure 19
	B3, _3_4, C4, _1_4,																					// Measure 20
	Du2, G4, _1_64, Du1, G4, _39_64, E4, _1_8, G4, _1_8, E4, _1_8,										// Measure 21
	D4, _1_8, E4, _1_16, A3, _7_16, C4, _1_8, G4, _1_8, E4, _1_8,										// Measure 22
	D4, _1_8, E4, _1_16, A3, _7_16, C4, _1_8, G4, _1_8, C4, _1_8,										// Measure 23
	B3, _3_4, C4, _1_4,																					// Measure 24
	Du2, C4, _1_8, A3, _1_16, C4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _1_8, A3, _3_16,			// Measure 25
	C4, _1_8, A3, _1_16, C4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _3_16, R, _1_8,					// Measure 26
	C4, _1_8, A3, _1_16, C4, _1_8, D4, _3_16, E4, _1_16, D4, _1_8, C4, _1_8, G3, _3_16,					// Measure 27
	C4, _1_8, G3, _1_16, C4, _1_8, D4, _3_16, E4, _1_8, R, _3_16, Sw, E4, G4, _3_16,					// Measure 28
	G4, _1_8, E4, _1_16, E4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _3_16, G4, _1_8,					// Measure 29
	G4, _1_8, A4, _1_8, Sw, A4, C5, _1_8, A4, _1_8, Sw, A4, C5, _1_8, A4, _1_16, A4, _3_16, G4, _1_8,	// Measure 30
	G4, _1_8, E4, _1_16, E4, _3_16, D4, _1_8, E4, _1_8, D4, _1_16, C4, _3_16, R, _1_8,					// Measure 31
	G4, _1_8, E4, _1_16, E4, _1_8, D4, _3_16, Sw, D4, E4, _1_4, G4, _1_4,								// Measure 32
	Sw, D5, E5, _1_8, D5, _1_8, E5, _1_8, G4, _3_16, A4, _1_16, R, _1_8, C5, _1_8, E5, _1_8,			// Measure 33
	D5, _1_16, R, _1_16, E5, _1_16, A4, _5_16, R, _1_4, G4, _1_4,										// Measure 34
	Sw, D5, E5, _1_8, D5, _1_8, E5, _1_8, G4, _3_16, A4, _1_16, R, _1_8, C5, _1_8, E5, _1_8,			// Measure 35
	D5, _1_16, R, _1_16, E5, _1_16, B4, _5_16, R, _1_8, G4, _1_8, G4, _1_16, G4, _3_16,					// Measure 36
	Sw, D5, E5, _1_8, D5, _1_8, E5, _1_8, G4, _3_16, A4, _1_16, R, _1_8, C5, _1_8, E5, _1_16, R, _1_16,	// Measure 37
	D5, _1_16, R, _1_16, E5, _1_16, A4, _5_16, R, _1_4, G4, _1_4,										// Measure 38
	Sw, D5, E5, _1_8, D5, _1_8, E5, _1_8, G4, _3_16, A4, _1_16, R, _1_8, C5, _1_8, E5, _1_16, R, _1_16,	// Measure 39
	D5, _1_16, R, _1_16, E5, _1_16, B4, _5_16, R, _1_8, G4, _1_8, G4, _1_16, G4, _3_16,					// Measure 40
	NULL
};
const uint16_t bruises_triangle[] =
{
	R, _1_1,																									// Intro
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 1
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 2
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 3
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,			// Measure 4
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 5
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 6
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 7
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,			// Measure 8
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 9
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 10
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 11
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,		 	// Measure 12
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 13
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 14
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 15
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,		 	// Measure 16
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 17
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 18
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 19
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,		 	// Measure 20
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 21
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 22
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 23
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,		 	// Measure 24
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 25
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 26
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 27
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,		 	// Measure 28
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 29
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 30
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 31
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,			// Measure 32
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 33
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 34
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 35
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,			// Measure 36
	C2, _1_8, R, _1_8, C2, _1_8, R, _1_8, C2, _1_8, R, _1_16, C2, _1_16, R, _1_16, C2, _1_16, G2, _1_8,			// Measure 37
	F2, _1_8, R, _1_8, F2, _1_8, R, _1_8, F2, _1_8, R, _1_16, F2, _1_16, R, _1_16, F2, _1_16, E2, _1_8,			// Measure 38
	A1, _1_8, R, _1_8, A1, _1_8, R, _1_8, A1, _1_8, R, _1_16, A1, _1_16, R, _1_16, A1, _1_16, C2, _1_8,			// Measure 39
	G1, _1_8, R, _1_8, G1, _1_8, R, _1_8, G1, _1_8, R, _1_16, G1, _1_16, R, _1_16, G1, _1_16, B1, _1_8,		 	// Measure 40
	NULL
};
const uint16_t bruises_noise[] =
{
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Intro
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 1
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 2
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 3
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 4
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 5
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 6
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 7
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 8
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 9
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 10
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 11
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 12
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 13
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 14
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 15
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 16
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 17
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 18
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 19
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 20
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 21
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 22
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 23
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 24
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 25
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 26
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 27
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 28
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 29
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 30
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 31
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 32
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 33
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 34
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 35
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 36
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 37
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 38
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 39
	R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _3_32, R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _3_32,			// Measure 40
	NULL
};
const uint16_t bruises_dpcm[] =
{
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Intro
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 1
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 2
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 3
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 4
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 5
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 6
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 7
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 8
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 9
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 10
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 11
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 12
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 13
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 14
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 15
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 16
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 17
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 18
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 19
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 20
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 21
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 22
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 23
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 24
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 25
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 26
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 27
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 28
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 29
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 30
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 31
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 32
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 33
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 34
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 35
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 36
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 37
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 38
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 39
	Kick, _1_4, Kick, _1_4, Kick, _3_16, Kick, _5_16,		// Measure 40
	NULL
};
const uint16_t faye_pulse1[] =
{
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	Sw, Cs4, E4, _1_4, E4, _1_8, Cs4, _1_8, B3, _1_4, Cs4, _1_8, E4, _1_8,
	E4, _1_4, R, _3_4,
	Sw, Cs4, E4, _1_4, E4, _1_8, Cs4, _1_8, B3, _1_4, E4, _1_8, Fs4, _1_8,
	Fs4, _1_4, R, _3_4,
	Gs4, _1_4, E4, _1_8, Cs4, _1_8, Gs4, _1_4, E4, _1_8, Cs4, _1_8,
	Fs4, _3_8, E4, _1_8, Cs4, _1_4, R, _1_4,
	A4, _1_4, Sw, A4, E4, _1_4, Cs4, _1_8, E4, _1_4, Fs4, _1_8,
	Fs4, _1_4, R, _3_4,
	Sw, Cs4, E4, _1_4, E4, _1_8, Cs4, _1_8, B3, _1_4, Cs4, _1_8, E4, _1_8,
	E4, _1_4, R, _3_4,
	Sw, Cs4, E4, _1_4, E4, _1_8, Cs4, _1_8, B3, _1_4, E4, _1_8, Fs4, _1_8,
	Fs4, _1_4, R, _3_4,
	Gs4, _1_4, E4, _1_8, Cs4, _1_8, Gs4, _1_4, E4, _1_8, Cs4, _1_8,
	Fs4, _3_8, E4, _1_8, Cs4, _1_4, R, _1_4,
	A4, _1_4, Sw, A4, E4, _1_4, Cs4, _1_8, E4, _1_4, Fs4, _1_8,
	Fs4, _1_4, R, _3_4,
	Fs4, _1_4, B3, _1_4, Ds4, _1_8, Cs4, _1_4, E4, _1_8,
	E4, _1_4, R, _3_4,
	R, _1_1,
	Sw, Fs4, Gs4, _1_8, Sw, Gs4, Fs4, _1_8, Sw, Fs4, Gs4, _1_8, Sw, Gs4, Fs4, _1_8, Gs4, _1_4, E4, _1_4,
	A4, _1_4, B4, _1_8, A4, _1_8, Gs4, _1_8, Fs4, _1_8, E4, _1_4,
	Fs4, _1_4, R, _1_2, Sw, B4, Cs5, _1_8, B4, _1_8,
	Fs4, _3_4, R, _1_4,
	Sw, Fs4, Gs4, _1_8, Sw, Gs4, Fs4, _1_8, Sw, Fs4, Gs4, _1_8, Sw, Gs4, Fs4, _1_8, Gs4, _1_4, E4, _1_4,
	A4, _1_4, B4, _1_8, A4, _1_8, Gs4, _1_8, Fs4, _1_8, E4, _1_4,
	Fs4, _1_4, R, _1_2, Sw, B4, Cs5, _1_8, B4, _1_8,
	Fs4, _1_1,
	R, _1_1,
	NULL
};
const uint16_t faye_triangle[] =
{
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, B3, _1_4, B3, _1_8, E4, _1_8, Ds3, _1_8,
	Cs3, _1_8, Gs3, _1_8, Cs4, _1_8, Gs3, _1_4, Gs3, _1_8, Cs4, _1_8, B3, _1_8,
	A2, _1_8, E3, _1_8, Cs4, _1_8, A3, _1_4, E3, _1_8, Cs4, _1_8, R, _1_8,
	Fs3, _1_4, A3, _1_4, B3, _1_4, Ds4, _1_4,
	Cs3, _1_8, Gs3, _1_8, Cs4, _1_8, Gs3, _1_4, Gs3, _1_8, Cs4, _1_8, B2, _1_8,
	A2, _1_8, E3, _1_8, A3, _1_8, E3, _1_4, Cs4, _1_8, A3, _1_8, E3, _1_8,
	Fs2, _1_8, Cs3, _1_8, A3, _1_8, Fs3, _1_4, Cs3, _1_8, Fs3, _1_8, Fs2, _1_8,
	E4, _1_2, Ds4, _1_2,
	R, _1_1,
	R, _1_4, E4, _1_4, Ds4, _1_4, B3, _1_4,
	Cs4, _1_2, B3, _1_2,
	A3, _1_2, B3, _1_4, Cs4, _1_8, Ds4, _1_8,
	E4, _1_1,
	A3, _3_4, A3, _1_8, B3, _1_8,
	Cs4, _1_1,
	E4, _1_2, Ds4, _1_2,
	E4, _1_2, Fs4, _1_2,
	Gs4, _1_1,
	Gs4, _1_2, Fs4, _1_4, B4, _1_4,
	R, _1_8, E4, _1_8, E5, _1_8, E4, _1_8, Ds5, _1_8, E4, _1_8, Cs5, _1_8, E4, _1_8,
	B4, _1_4, A4, _1_4, Gs4, _1_4, E4, _1_4,
	R, _1_8, Cs4, _1_8, Cs5, _1_8, Cs4, _1_8, B4, _1_8, Cs4, _1_8, A4, _1_8, Cs4, _1_8,
	Gs4, _1_4, Fs4, _1_4, Gs4, _1_4, A4, _1_4,
	R, _1_8, E4, _1_8, E5, _1_8, E4, _1_8, Ds5, _1_8, E4, _1_8, Cs5, _1_8, E4, _1_8,
	B4, _1_4, A4, _1_4, Gs4, _1_4, E4, _1_4,
	R, _1_8, A3, _1_8, A4, _1_8, A3, _1_8, Gs4, _1_8, A3, _1_8, Fs4, _1_8, A3, _1_8,
	E4, _1_4, Ds4, _1_4, E4, _1_4, Fs4, _1_4,
	B4, _1_1,
	NULL
};
/*
const uint16_t faye_triangle[] =
{
	E2, _1_8, B2, _1_8, E3, _1_8, R, _1_4, E3, _1_8, B3, _1_8, Fs3, _1_8,
	E2, _1_8, B2, _1_8, E3, _1_8, R, _1_4, E3, _1_8, B3, _1_8, Fs3, _1_8,
	E2, _1_8, B2, _1_8, E3, _1_8, R, _1_4, E3, _1_8, B3, _1_8, Fs3, _1_8,
	E2, _1_8, B2, _1_8, E3, _1_8, R, _1_4, E3, _1_8, B3, _1_8, Fs3, _1_8,
	E2, _1_8, B2, _1_8, E3, _1_8, B2, _1_4, B2, _1_8, E3, _1_8, Ds2, _1_8,
	Cs2, _1_8, Gs2, _1_8, Cs3, _1_8, Gs2, _1_4, Gs2, _1_8, Cs3, _1_8, B2, _1_8,
	A1, _1_8, E2, _1_8, Cs3, _1_8, A2, _1_4, E2, _1_8, Cs3, _1_8, R, _1_8,
	Fs2, _1_4, A2, _1_4, B2, _1_4, Ds3, _1_4,
	Cs2, _1_8, Gs2, _1_8, Cs3, _1_8, Gs2, _1_4, Gs2, _1_8, Cs3, _1_8, B1, _1_8,
	A1, _1_8, E2, _1_8, A2, _1_8, E2, _1_4, Cs3, _1_8, A2, _1_8, E2, _1_8,
	Fs1, _1_8, Cs2, _1_8, A2, _1_8, Fs2, _1_4, Cs2, _1_8, Fs2, _1_8, Fs1, _1_8,
	E3, _1_2, Ds3, _1_2,
	R, _1_1,
	R, _1_4, E3, _1_4, Ds3, _1_4, B2, _1_4,
	Cs3, _1_2, B2, _1_2,
	A2, _1_2, B2, _1_4, Cs3, _1_8, Ds3, _1_8,
	E3, _1_1,
	A2, _3_4, A2, _1_8, B2, _1_8,
	Cs3, _1_1,
	E3, _1_2, Ds3, _1_2,
	E3, _1_2, Fs3, _1_2,
	Gs3, _1_1,
	Gs3, _1_2, Fs3, _1_4, B3, _1_4,
	R, _1_8, E3, _1_8, E4, _1_8, E3, _1_8, Ds4, _1_8, E3, _1_8, Cs4, _1_8, E3, _1_8,
	B4, _1_4, A4, _1_4, Gs4, _1_4, E4, _1_4,
	R, _1_8, Cs3, _1_8, Cs4, _1_8, Cs3, _1_8, B3, _1_8, Cs3, _1_8, A3, _1_8, Cs3, _1_8,
	Gs3, _1_4, Fs3, _1_4, Gs3, _1_4, A3, _1_4,
	R, _1_8, E4, _1_8, E4, _1_8, E4, _1_8, Ds4, _1_8, E3, _1_8, Cs4, _1_8, E3, _1_8,
	B3, _1_4, A3, _1_4, Gs3, _1_4, E3, _1_4,
	R, _1_8, A2, _1_8, A3, _1_8, A2, _1_8, Gs3, _1_8, A2, _1_8, Fs3, _1_8, A2, _1_8,
	E3, _1_4, Ds3, _1_4, E3, _1_4, Fs3, _1_4,
	B3, _1_1,
	NULL
};
*/
const uint16_t faye_noise[] =
{
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,	
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	NULL
};
const uint16_t faye_dpcm[] =
{
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,	
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	R, _1_1,
	NULL	
};

const uint16_t phoenix_pulse1[] =
{
	R, _1_8, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, A3, _1_8, Du2, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, A3, _1_8,								// Measure 1
	R, _1_8, Du2, Gs3, _1_8, Du1, Gs3, _1_16, Du2, Gs3, _1_16, Du1, Gs3, _1_8, Sw, Gs3, Fs3, _1_16, Fs3, _1_16, Du2, Fs3, _1_16, Du1, Fs3, _1_8, Sw, Fs3, A3, _1_8,
	R, _1_8, Du2, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, B3, _1_8, Du2, A3, _3_16, Gs3, _1_16, Du1, Gs3, _3_16,
	R, _1_8, Du2, Cs4, _1_8, Du1, Cs4, _1_8, Du2, Cs4, _1_16, Du1, E4, _1_4, Du2, Cs4, _1_16, E4, _1_8, Sw, E4, Fs4, _1_8,

	R, _1_8, A3, _1_8, Du1, A3, _1_16, Du0, A3, _1_8, Du1, A3, _1_8, Du2, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, A3, _1_8,								// Measure 5
	R, _1_8, Du2, A3, _1_8, Du1, A3, _1_16, Du2, A3, _1_8, Du1, B3, _1_8, Du2, A3, _1_16, Du1, A3, _1_16, Du2, A3, _1_16, Du1, A3, _1_8, Du2, Gs3, _1_8,
	R, _1_8, E4, _1_8, Cs4, _1_8, B3, _1_16, Du1, B3, _1_16, Du2, B3, _1_8, Du1, B3, _1_16, Du2, B3, _3_16, Sw, B3, Cs4, _1_8,
		
	/*
	R, _1_8, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, A3, _1_8, Du2, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, A3, _1_8,								// Measure 8
	R, _1_8, Du2, Gs3, _1_8, Du1, Gs3, _1_16, Du0, Gs3, _1_16, Du1, Gs3, _1_8, Du2, Fs3, _1_16, Du1, Fs3, _1_16, Du0, Fs3, _1_16, Du1, Fs3, _1_8, Sw, Fs3, A3, _1_8,
	R, _1_8, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, B3, _1_8, Du2, A3, _3_16, Gs3, _1_16, Du1, Gs3, _3_16,
	R, _1_8, Du2, E4, _1_64, Du1, E4, _3_64, Du2, E4, _1_64, Du1, E4, _3_64, Du2, Sw, E4, Cs4, _1_8, Cs4, _1_16, Du0, Cs4, _1_16, Du2, Sw, Cs4, B3, _1_8, B3, _1_8, B3, _1_8, Cs4, _1_8,
		
	R, _1_8, Cs4, _1_8, Cs4, _1_16, Cs4, _1_8, E4, _1_8, Cs4, _1_8, E4, _1_8, Cs4, _1_16, Fs4, _1_8,						// Measure 12
	R, _1_8, A3, _1_16, A3, _1_16, A3, _1_16, A3, _1_8, A3, _1_8, Fs3, _1_8, Fs3, _1_8, E3, _1_16, Fs3, _1_8,
	R, _1_8, A3, _1_8, A3, _1_16, A3, _1_8, B3, _1_8, A3, _1_8, Gs3, _1_8, A3, _1_16, Gs3, _1_8,
	R, _1_8, E4, _1_16, E4, _1_16, E4, _1_8, Cs4, _1_16, Cs4, _1_16, Cs4, _1_8, B3, _1_16, B3, _1_16, B3, _1_8, Cs4, _1_8,
		
	R, _1_8, Cs4, _1_8, Cs4, _1_16, Cs4, _1_8, E4, _1_4, Cs4, _1_16, E4, _1_8, Fs4, _1_8,									// Measure 16
	R, _1_8, A3, _1_16, A3, _1_16, A3, _1_4, A3, _1_16, A3, _1_8, A3, _1_8, A3, _1_16, A3, _1_8,
	R, _1_8, A3, _1_16, A3, _1_16, A3, _1_8, A3, _1_8, B3, _1_8, A3, _1_8, Gs3, _1_8, Gs3, _1_8,
	R, _1_8, E4, _1_16, E4, _1_16, E4, _1_8, Cs4, _1_16, Cs4, _1_16, Cs4, _1_8, B3, _1_16, B3, _1_16, B3, _1_8, Cs4, _1_8,
		
	R, _1_8, Cs4, _1_8, Cs4, _1_16, Cs4, _1_8, E4, _1_8, Cs4, _1_8, E4, _1_8, Cs4, _1_16, Fs4, _1_8,						// Measure 20
	R, _1_8, A3, _1_16, A3, _1_16, A3, _1_8, Gs3, _1_16, Gs3, _1_16, Gs3, _1_16, Fs3, _1_8, Fs3, _1_8, E3, _1_16, Fs3, _1_8,
	R, _1_8, A3, _1_8, A3, _1_16, A3, _1_16, A3, _1_16, B3, _1_8, A3, _1_16, A3, _1_16, A3, _1_16, Gs3, _1_8, Gs3, _1_8,
	R, _1_8, E4, _1_16, E4, _1_16, Cs4, _1_8, Cs4, _1_16, Cs4, _1_16, B3, _1_8, B3, _1_16, B3, _1_16, A3, _1_8, A3, _1_16, A3, _1_16,

	R, _1_4, Fs3, _1_8, Fs3, _1_8, B3, _1_8, B3, _1_8, A3, _1_4,															// Measure 24
	Gs3, _1_8, A3, _1_8, Gs3, _1_2, R, _1_4,
	R, _1_8, Fs3, _1_8, Fs3, _1_8, Fs3, _1_8, B3, _1_8, B3, _1_8, A3, _1_8, A3, _1_8,
	Gs3, _1_8, A3, _1_16, Gs3, _5_16, R, _1_2,
			
	R, _1_4, Fs3, _1_8, Fs3, _1_8, B3, _1_8, B3, _1_8, A3, _1_4,															// Measure 28
	E4, _1_8, Cs4, _5_8, R, _1_4,
	R, _1_8, E4, _1_8, Cs4, _1_8, B3, _1_8, B3, _1_8, Cs4, _1_8, B3, _1_4,
	B3, _1_8, B3, _1_16, B3, _1_8, B3, _1_2, R, _3_16,
	*/
	NULL
};
const uint16_t phoenix_triangle[] =
{
	D2, _1_8, D2, _1_8, R, _3_4,
	Cs2, _1_4, R, _3_4,
	E2, _1_8, E2, _1_8, R, _3_4,
	D2, _1_8, D2, _1_8, R, _3_4,
	Cs2, _1_4, R, _3_4,
	Fs1, _1_1,
	Fs2, _1_8, Fs2, _1_8, F2, _1_8, F2, _1_8, E2, _1_8, E2, _1_8, Sw, E2, A2, _1_4,
	
	/*
	// 8
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_8, Cs2, _1_8, R, _1_8, Cs2, _1_8,
	Fs1, _1_8, Fs1, _1_8, R, _1_4, A1, _1_4, Cs2, _1_4,
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_4, R, _1_8, E2, _1_8,
	Fs2, _1_4, R, _1_8, E2, _1_8, R, _1_8, B1, _1_8, Cs2, _1_4,
	
	// 12
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_8, Cs2, _1_8, R, _1_8, Cs2, _1_8,
	Fs1, _1_8, Fs1, _1_8, R, _1_4, A1, _1_4, Cs2, _1_4,
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_4, R, _1_8, E2, _1_8,
	Fs2, _1_4, R, _1_8, E2, _1_8, R, _1_8, B1, _1_8, Cs2, _1_4,
	
	// 16
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_8, Cs2, _1_8, R, _1_8, Cs2, _1_8,
	Fs1, _1_8, Fs1, _1_8, R, _1_4, A1, _1_4, Cs2, _1_4,
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_4, R, _1_8, E2, _1_8,
	Fs2, _1_4, R, _1_8, E2, _1_8, R, _1_8, B1, _1_8, Cs2, _1_4,
	
	// 20
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_8, Cs2, _1_8, R, _1_8, Cs2, _1_8,
	Fs1, _1_8, Fs1, _1_8, R, _1_4, A1, _1_4, Cs2, _1_4,
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_4, R, _1_8, E2, _1_8,
	Fs2, _1_4, R, _1_8, E2, _1_8, R, _1_8, B1, _1_8, Cs2, _1_4,
	
	// 24
	Fs1, _1_8, Fs1, _1_8, R, _1_8, Fs1, _1_8, B1, _1_8, B1, _1_8, R, _1_8, B1, _1_8,
	A1, _1_8, A1, _1_8, R, _1_8, A1, _1_8, D2, _1_4, Cs2, _1_4,
	D2, _1_8, D3, _1_8, D2, _1_8, Ds2, _1_8, E2, _1_8, E3, _1_8, E2, _1_8, F2, _1_8,
	Fs2, _1_8, Fs2, _1_8, E2, _1_8, F2, _1_8, Fs2, _1_8, R, _1_8, A2, _1_4,
	
	// 28
	Fs1, _1_8, Fs1, _1_8, R, _1_8, Fs1, _1_8, B1, _1_8, B1, _1_8, R, _1_8, B1, _1_8,
	A1, _1_8, A1, _1_8, R, _1_8, A1, _1_8, D2, _1_4, Cs2, _1_4,
	D2, _1_8, D3, _1_8, D2, _1_8, Ds2, _1_8, E2, _1_8, E3, _1_8, E2, _1_8, F2, _1_8,
	Fs2, _1_8, Fs2, _1_8, E2, _1_8, F2, _1_8, Fs2, _1_8, R, _1_8, A2, _1_4,	
	*/
	
	NULL
};
const uint16_t phoenix_noise[] =
{
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,	
	NULL
};
const uint16_t phoenix_dpcm[] =
{
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	NULL
};

/*
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, R, _1_4, E4, _1_8, B4, _1_8, Fs4, _1_8,
	E3, _1_8, B3, _1_8, E4, _1_8, B3, _1_4, B3, _1_8, E4, _1_8, Ds3, _1_8,
	Cs3, _1_8, Gs3, _1_8, Cs4, _1_8, Gs3, _1_4, Gs3, _1_8, Cs4, _1_8, B3, _1_8,
	A2, _1_8, E3, _1_8, Cs4, _1_8, A3, _1_4, E3, _1_8, Cs4, _1_8, R, _1_8,
	Fs3, _1_4, A3, _1_4, B3, _1_4, Ds4, _1_4,
	Cs3, _1_8, Gs3, _1_8, Cs4, _1_8, Gs3, _1_4, Gs3, _1_8, Cs4, _1_8, B2, _1_8,
	A2, _1_8, E3, _1_8, A3, _1_8, E3, _1_4, Cs4, _1_8, A3, _1_8, E3, _1_8,
	Fs2, _1_8, Cs3, _1_8, A3, _1_8, Fs3, _1_4, Cs3, _1_8, Fs3, _1_8, Fs2, _1_8,
	E4, _1_2, Ds4, _1_2,
	R, _1_1,
	R, _1_4, E4, _1_4, Ds4, _1_4, B3, _1_4,	
	Cs4, _1_2, B3, _1_2,
	A3, _1_2, B3, _1_4, Cs4, _1_8, Ds4, _1_8,
	E4, _1_1,
	A3, _3_4, A3, _1_8, B3, _1_8,
	Cs4, _1_1,
	E4, _1_2, Ds4, _1_2,
	E4, _1_2, Fs4, _1_2,
	Gs4, _1_1,
	Gs4, _1_2, Fs4, _1_4, B4, _1_4,
	R, _1_8, E4, _1_8, E5, _1_8, E4, _1_8, Ds5, _1_8, E4, _1_8, Cs5, _1_8, E4, _1_8,
	B4, _1_4, A4, _1_4, Gs4, _1_4, E4, _1_4,
	R, _1_8, Cs4, _1_8, Cs5, _1_8, Cs4, _1_8, B4, _1_8, Cs4, _1_8, A4, _1_8, Cs4, _1_8,
	Gs4, _1_4, Fs4, _1_4, Gs4, _1_4, A4, _1_4,
	R, _1_8, E4, _1_8, E5, _1_8, E4, _1_8, Ds5, _1_8, E4, _1_8, Cs5, _1_8, E4, _1_8,
	B4, _1_4, A4, _1_4, Gs4, _1_4, E4, _1_4,
	R, _1_8, A3, _1_8, A4, _1_8, A3, _1_8, Gs4, _1_8, A3, _1_8, Fs4, _1_8, A3, _1_8,
	E4, _1_4, Ds4, _1_4, E4, _1_4, Fs4, _1_4,
	B4, _1_1,	
*/
/* // Phoenix DPCM
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	
	// Phoenix Noise
	R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8,
	R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8,
	R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8,
	R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8, R, _1_8, H, _1_8,
	
	// Phoenix Bass
	

*/
const uint16_t test_pulse1[] =
{
	R, _1_8, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, A3, _1_8, Du2, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, A3, _1_8,								// Measure 8
	R, _1_8, Du2, Gs3, _1_8, Du1, Gs3, _1_16, Du0, Gs3, _1_16, Du1, Gs3, _1_8, Du2, Fs3, _1_16, Du1, Fs3, _1_16, Du0, Fs3, _1_16, Du1, Fs3, _1_8, Sw, Fs3, A3, _1_8,
	R, _1_8, A3, _1_8, Du1, A3, _1_8, Du0, A3, _1_16, Du1, B3, _1_8, Du2, A3, _3_16, Gs3, _1_16, Du1, Gs3, _3_16,
	R, _1_8, Du2, E4, _1_64, Du1, E4, _3_64, Du2, E4, _1_64, Du1, E4, _3_64, Du2, Sw, E4, Cs4, _1_8, Cs4, _1_16, Du0, Cs4, _1_16, Du2, Sw, Cs4, B3, _1_8, B3, _1_8, B3, _1_8, Cs4, _1_8,
	NULL
};
const uint16_t test_triangle[] =
{
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_8, Cs2, _1_8, R, _1_8, Cs2, _1_8,
	Fs1, _1_8, Fs1, _1_8, R, _1_4, A1, _1_4, Cs2, _1_4,
	D2, _1_8, D2, _1_8, R, _1_8, D2, _1_8, E2, _1_4, R, _1_8, E2, _1_8,
	Fs2, _1_4, R, _1_8, E2, _1_8, R, _1_8, B1, _1_8, Cs2, _1_4,
	NULL
};
const uint16_t test_noise[] =
{
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,
	R, _1_8, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _7_32, H, _1_32, R, _3_32,	
	NULL
};
const uint16_t test_dpcm[] =
{
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	Kick, _1_4, Rim, _1_4, Kick, _1_4, Rim, _1_4,
	NULL
};

//
// Button press debounce routines
//
ISR (INT0_vect) // PD2
{
	if ( !s_pressed )
	{
		s_pressed = 1;
		s_debounce_time = 100;
	}
}
ISR (INT1_vect) // PD3
{
	if ( !u_pressed )
	{
		u_pressed = 1;
		u_debounce_time = 100;
	}
}
ISR (INT2_vect) // PB2
{
	if ( !d_pressed) {
		d_pressed = 1;
		d_debounce_time = 100;
	}
}

//
// DDS for pulse channel 1; 1-millisecond utility timer
//
ISR (TIMER0_OVF_vect)
{
	if ( juke_go )
	{
		if ( pulse_go )
		{
			if ( !p1_silence )
			{
				if ( s_go )
				{
					// Sweep p1 frequency at 120 Hz
					if ( s_divider > 0 ) --s_divider;
					else
					{
						// Determine sweep increment to add/subtract
						s_increment = p1_increment >> SHIFT_COUNT;
					
						// Desired frequency is above starting note
						if ( s_up )
						{
							// Increase p1 frequency until target note is reached
							if ( p1_increment + s_increment < s_des_increment ) {
								p1_increment = p1_increment + s_increment;
							}
							else {
								p1_increment = s_des_increment;
								s_go = 0;
							}
						}
					
						// Desired frequency is below starting note	
						else
						{
							// Decrease p1 frequency until target note is reached
							if ( p1_increment - s_increment > s_des_increment ) {
								p1_increment = p1_increment - s_increment;
							}
							else {
								p1_increment = s_des_increment;
								s_go = 0;
							}
						}
					
						s_divider = 240;
					
					} // end sweep update
				
				} // end sweep whole routine
					
				// Step through pulse waveform sequence
				p1_accumulator = p1_accumulator + p1_increment;
				p1_sequencer = p1_accumulator >> 13;
		
				if ( p1_duty == 0 ) {
					OCR0A = P1_OUTPUT_LEVEL * p1_duty0[p1_sequencer];
				}
				else if ( p1_duty == 1 ) {
					OCR0A = P1_OUTPUT_LEVEL * p1_duty1[p1_sequencer];
				}
				else if ( p1_duty == 2 ) {
					OCR0A = P1_OUTPUT_LEVEL * p1_duty2[p1_sequencer];
				}
				else {
					OCR0A = P1_OUTPUT_LEVEL * p1_duty3[p1_sequencer];
				}
				
			} // end pulse1 routine
		
			pulse_go ^= 1;

		} // end pulse_go routine

		else
		{
			if ( !t_silence )
			{
				if ( b_go )
				{
					// Sweep t frequency at 120 Hz
					if ( b_divider > 0 ) --b_divider;
					else
					{
						// Determine sweep increment to add/subtract
						b_increment = t_increment >> SHIFT_COUNT;
									
						// Desired frequency is above starting note
						if ( b_up )
						{
							// Increase t frequency until target note is reached
							if ( t_increment + b_increment < b_des_increment ) {
								t_increment = t_increment + b_increment;
							}
							else {
								t_increment = b_des_increment;
								b_go = 0;
							}
						}
									
						// Desired frequency is below starting note
						else
						{
							// Decrease t frequency until target note is reached
							if ( t_increment - b_increment > b_des_increment ) {
								t_increment = t_increment - b_increment;
							}
							else {
								t_increment = b_des_increment;
								b_go = 0;
							}
						}
									
						b_divider = 240;
									
					} // end sweep update
								
				} // end sweep whole routine
							
				t_accumulator = t_accumulator + t_increment;
				t_sequencer = t_accumulator >> 11;
				OCR0B = 16 * triangle[t_sequencer];
			}
			
			if ( !n_silence )
			{
				uint8_t feedback = noise & 1 ^ (noise >> 1) & 1;
			
				noise = noise >> 1;
			
				noise = noise + (feedback << 14);
			
				OCR1A = (unsigned short)(255) * (noise & 1 ^ 1);
			}
		
			if ( !d_silence )
			{
				d_last_output = d_output;
				
				// Add or subtract 2 from output level
				if ( d_register & 1 ) {
					if ( d_last_output < 126 ) {
						d_output = d_last_output + 2;
					}
				}
				else {
					if ( d_last_output > 1 ) {
						d_output = d_last_output - 2;
					}
				}
				
				OCR1B = 2 * d_output;
			
				// Shift register, decrement remaining bits
				d_register = d_register >> 1;
				if ( d_bits_rem > 0 ) d_bits_rem--;
			
				// If remaining bits is zero
				if ( d_bits_rem == 0 ) {
					
					// Reset bits-remaining
					d_bits_rem = 8;
				
					// If sample buffer is empty, set silence flag
					if ( *(d_sample + 1) == NULL_SAMP )
					{				
						// Silence until next note
						d_silence = 1;
					}
					// Else, move onto next sample byte
					else {	
						d_register = *(++d_sample);
					}
				}
			
			} // end DPCM branch
		
			pulse_go ^= 1;
		
		} // end !pulse_go routine

	} // end juke_go routine

	
	// Generate 1-millisecond time base
	if ( base_time > 0 ) --base_time;
	if ( !base_time ) {
		base_time = TICKS_PER_MS;
		if ( p1_length > 0 ) --p1_length;
		else p1_ready = 1;
		if ( t_length > 0 ) --t_length;
		else t_ready = 1;
		if ( n_length > 0 ) --n_length;
		else n_ready = 1;
		if ( d_length > 0 ) --d_length;
		else d_ready = 1;
		if ( u_debounce_time > 0 ) --u_debounce_time;
		if ( d_debounce_time > 0 ) --d_debounce_time;
		if ( s_debounce_time > 0 ) --s_debounce_time;
	}
}

//
// Initialize LCD
//
void init_lcd(void)
{
	LCDinit(); // initialize the display
	LCDcursorOn(); // turn off visibility of cursor
	LCDclr(); // clear the display
	LCDGotoXY(0,0); // position cursor
}

//
// Initialize timers, ports, LCD
//
void initialize(void)
{
	// Initialize LED on PortB0 to on, for debugging
	DDRB |= (1 << PINB0);
	PORTB |= (1 << PINB0);
	
	// Set up LCD; Write out mode and song names
	init_lcd();
	CopyStringtoLCD(lcd_juke, 1, 0);
	CopyStringtoLCD(lcd_bruises, 0, 1);
	CopyStringtoLCD(lcd_forever, -4, 2);
	CopyStringtoLCD(lcd_phoenix, -4, 3);
	LCDGotoXY(0,0);
	
	// Set OC0A, OC0B, OC1A, and OC1B to output 
	DDRB |= (1 << PINB3) + (1 << PINB4); // pulse1, triangle channel outputs
	DDRD |= (1 << PIND4) + (1 << PIND5); // noise, DPCM channel outputs
	
	// Set up data bus on PORTA
	DDRA = 0;

	// Set up push buttons
	DDRB &= ~(1 << PINB2);
	PORTB |= (1 << PINB2);
	DDRD &= ~(1 << PIND3) + ~(1 << PIND2);
	PORTD |= (1 << PIND3) + (1 << PIND2);
	EICRA |= (1 << ISC01) + (1 << ISC11) + (1 << ISC21); // trigger interrupt on falling edge
	EIMSK |= (1 << INT0) + (1 << INT1) + (1 << INT2);
	
	// Initialize ramp table
	/*
	for (int i = 0; i < 256; i++)
	{
		// Ramp values range from 0 to 127
		ramp[i] = (i >> 1);
	}
	*/
	
	// Timer 0 setup: pulse1 and triangle channel outputs, DDS for all channels, 1-millisecond utility timer
	TCCR0A |= (1 << WGM00) + (1 << WGM01) + (1 << COM0A1) + (1 << COM0B1); // fast PWM; toggle OC0A and OC0B in non-inverting mode
	TCCR0B |= (1 << CS00); // full-speed
	TIMSK0 |= (1 << TOIE0); // enable Timer0 overflow ISR
	OCR0A = 128;
	OCR0B = 128;
	
	// Timer 1 setup: noise and DPCM channel outputs
	TCCR1A |= (1 << WGM10) + (1 << COM1A1) + (1 << COM1B1); // fast PWM, 8-bit; toggle OC1A in non-inverting mode
	TCCR1B |= (1 << WGM12) + (1 << CS10); // fast PWM, 8-bit; full-speed
	OCR1A = 128;
	OCR1B = 128;

	// Enable interrupts
	sei();
}

//
// Parse through and play tracks
//
void juke(const uint16_t* track1, const uint16_t* track2, const uint16_t* track3, const uint16_t* track4)
{
	// Assign pointers for each track
	const uint16_t* p1 = track1;
	const uint16_t* t = track2;
	const uint16_t* n = track3;
	const uint16_t* d = track4;
	
	// Note variables
	uint16_t note = 0;
	uint16_t note_c = 0;
	uint8_t dur_top = 0;
	uint8_t dur_bot = 0;
		
	// Current position of tracks (in 384ths of a beat)
	uint32_t p1_position = 0;
	uint32_t t_position = 0;
	uint32_t n_position = 0;
	uint32_t d_position = 0;
	
	// Loop through track note by note
	while ( (*p1 || *t || *n || *d) && juke_go )
	{
		// At the end of each measure
		if ( p1_position % 384 == 0 && t_position % 384 == 0 && n_position % 384 == 0 && d_position % 384 == 0 )
		{
			// If affective mode, set tempo according to measured bpm
			if ( mode == AFFECTIVE )
			{
				// Read heart rate value from PortA
				if ( PINA ) {
					bpm_desired = PINA;
				}
			
				// Adjust bpm_current either up or down; Enforce maximum/minimum bpm?
				if ( bpm_current != bpm_desired )
				{
					// Approach measured bpm logarithmically
					bpm_current = (bpm_desired + bpm_current) / 2;
					PORTB ^= (1 << PINB0);
					
					// Set beat duration according to new tempo
					beat_dur = 240000 / bpm_current;
				}
	
			} // end affective mode routine
			
			// Sync up tracks
			while( !p1_ready || !t_ready || !n_ready || !d_ready );
			
		} // end measure end routine
		
		// Parse through noise track if its current position is earliest
		if ( *n && n_position <= p1_position && n_position <= t_position && n_position <= d_position && n_ready )
		{
			// Get note frequency
			note = *n++;

			// Get note duration (top)
			dur_top = *n >> 8;

			// Get note duration (bottom)
			dur_bot = *n++ & 0x00ff;
			
			// Play the note
			if ( note > 1 )
			{
				// Determine duration of note in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
				
				// Hold note for duration
				n_length = duration;
				n_ready = 0;
				
				// Clear flags
				n_silence = 0;
			}
			
			// Otherwise, rest for duration
			else
			{
				// Determine duration of rest in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
				
				// Silence for duration
				n_length = duration;
				n_ready = 0;
				
				// Indicate that Timer0 ISR should skip PWM loop
				n_silence = 1;
			}
			
			// Update current position of pulse1 track, in 384ths of a beat
			n_position = n_position + 384 * dur_top / dur_bot;
			
		} // end noise branch
			
		// Parse through pulse1 track if its current position is earliest
		if ( *p1 && p1_position <= t_position && p1_position < n_position && p1_position <= d_position && p1_ready )
		{
			// Get note frequency
			note = *p1++;
			
			// Change duty cycle and play specified note
			if ( note > Sw && note < COMMAND)
			{
				// Set duty cycle
				p1_duty = note - 3;
				note = *p1++;
			}
							
			// Play the note
			if ( note > COMMAND )
			{
				// Get note duration (top)
				dur_top = *p1 >> 8;

				// Get note duration (bottom)
				dur_bot = *p1++ & 0x00ff;
								
				// Determine duration of note in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
				
				// Determine increment for stepping through 16-bit accumulator
				p1_increment = INC_SCALAR * note;
				
				// Clear flags
				p1_silence = 0;
								
				// Hold note for duration
				p1_length = duration;
				p1_ready = 0;
			}
			
			// Sweep from specified starting to ending note, and hold for duration
			else if ( note == Sw )
			{														
				// Set up sweep parameters
				p1_increment = INC_SCALAR * *p1++; // start playing note
				s_des_increment = INC_SCALAR * *p1++;
				if ( p1_increment < s_des_increment ) s_up = 1;
				else s_up = 0;
				
				// Get note duration (top)
				dur_top = *p1 >> 8;

				// Get note duration (bottom)
				dur_bot = *p1++ & 0x00ff;
					
				// Determine duration of note in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
				
				// Start sweep; clear flags
				s_go = 1;
				p1_silence = 0;
				
				// Hold note for duration
				p1_length = duration; // start counting down
				p1_ready = 0;
			}
			
			// Otherwise, rest for duration
			else
			{
				// Get note duration (top)
				dur_top = *p1 >> 8;

				// Get note duration (bottom)
				dur_bot = *p1++ & 0x00ff;
								
				// Determine duration of rest in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
					
				// Silence for duration
				p1_length = duration;
				p1_ready = 0;
					
				// Indicate that Timer0 ISR should skip PWM loop
				p1_silence = 1;
			}
				
			// Update current position of pulse1 track, in 384ths of a beat
			p1_position = p1_position + 384 * dur_top / dur_bot;
				
		} // end pulse1 branch
		
		// Parse through triangle track if its current position is earliest
		if ( *t && t_position < p1_position && t_position < n_position && t_position <= d_position && t_ready )
		{
			// Get note frequency
			note = *t++;
					
			// Play the note
			if ( note > COMMAND )
			{
				// Get note duration (top)
				dur_top = *t >> 8;

				// Get note duration (bottom)
				dur_bot = *t++ & 0x00ff;
				
				// Determine increment for stepping through 16-bit accumulator
				t_increment = INC_SCALAR * note;
						
				// Determine duration of note in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
						
				// Hold note for duration
				t_length = duration;
				t_ready = 0;
						
				// Clear flags
				t_silence = 0;
			}

			else if ( note == Sw )
			{
				// Set up sweep parameters
				b_increment = INC_SCALAR * *t++; // start playing note
				b_des_increment = INC_SCALAR * *t++;
				if ( t_increment < b_des_increment ) b_up = 1;
				else b_up = 0;
				
				// Get note duration (top)
				dur_top = *t >> 8;

				// Get note duration (bottom)
				dur_bot = *t++ & 0x00ff;
				
				// Determine duration of note in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
				
				// Start sweep; clear flags
				b_go = 1;
				t_silence = 0;
				
				// Hold note for duration
				t_length = duration; // start counting down
				t_ready = 0;
			}
					
			// Otherwise, rest for duration
			else
			{
				// Get note duration (top)
				dur_top = *t >> 8;

				// Get note duration (bottom)
				dur_bot = *t++ & 0x00ff;
				
				// Determine duration of rest in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
						
				// Silence for duration
				t_length = duration;
				t_ready = 0;
						
				// Indicate that Timer0 ISR should skip PWM loop
				t_silence = 1;
			}
					
			// Update current position of pulse1 track, in 384ths of a beat
			t_position = t_position + 384 * dur_top / dur_bot;
					
		} // end triangle branch
		
		// Parse through DPCM track if its current position is earliest
		if ( *d && d_position < p1_position && d_position < t_position && d_position < n_position && d_ready )
		{
			// Get note frequency
			note = *d++;

			// Get note duration (top)
			dur_top = *d >> 8;

			// Get note duration (bottom)
			dur_bot = *d++ & 0x00ff;
			
			// Play the note
			if ( note > 1 )
			{
				// Determine duration of note in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
				
				// Load new sample
				if ( note == Kick ) {
					d_sample = mario_bass_kick;
				}
				else if ( note == Rim ) {
					d_sample = tecmo_rim_shot;
				} 
				d_register = *d_sample;
				d_output = 64;
				
				// Hold note for duration
				d_length = duration;
				d_ready = 0;
				
				// Play sample
				d_silence = 0;
			}
			
			// Otherwise, rest for duration
			else
			{
				// Determine duration of rest in milliseconds
				uint16_t duration = beat_dur * dur_top / dur_bot;
				
				// Silence for duration
				d_length = duration;
				d_ready = 0;
				
				// Indicate that Timer0 ISR should skip PWM loop
				d_silence = 1;
			}
			
			// Update current position of pulse1 track, in 384ths of a beat
			d_position = d_position + 384 * dur_top / dur_bot;
			
		} // end DPCM branch

		// Read button press from user (select)
		if ( s_debounce_time == 0 && s_pressed )
		{
			s_pressed = 0;

			// Check if button is still pressed,
			if ( (PIND & (1 << PIND2)) == 0 ) {
				juke_go = 0;
			}
		}	

	} // end parser while loop

	// Finish playing this loop
	while( !p1_ready || !t_ready || !n_ready || !d_ready );
	
	// Silence channels before moving on
	p1_silence = 1;
	t_silence = 1;
	n_silence = 1;
	d_silence = 1;
	
} // end juke

//
// They say an end can be a start
//
void phoenix(void) 
{
	// Set tempo
	if (mode == JUKE) {
		bpm_current = 115;
	}
	else {
		bpm_current = 60;
	}
	
	beat_dur = 240000 / bpm_current;
	p1_duty = 2;
	
	// Play song
	while(juke_go) {
		juke(phoenix_pulse1, phoenix_triangle, phoenix_noise, phoenix_dpcm);
	}	
}
//
// Ming yue ji shi you
//
void faye(void)
{
	// Set tempo
	if (mode == JUKE) {
		bpm_current = 92;
	}
	else {
		bpm_current = 60;
	}
	
	beat_dur = 240000 / bpm_current;
	p1_duty = 2;
	
	// Play song
	while(juke_go) {
		juke(faye_pulse1, faye_triangle, faye_noise, faye_dpcm);
	}
}

//
// I tried to do handstands for you
//
void bruises(void)
{
	// Set tempo
	if ( mode == JUKE ) {
		bpm_current = 92;
	}
	else {
		bpm_current = 60;
	}
	beat_dur = 240000 / bpm_current;
	p1_duty = 2;
	
	// Play song
	while(juke_go) {
		juke(bruises_pulse1, bruises_triangle, bruises_noise, bruises_dpcm);
	}
}

//
// Test tune for debuggin
//
void test(void)
{
	// Set tempo
	bpm_current = 90;
	beat_dur = 240000 / bpm_current;
	
	// Play song
	while(juke_go) {
		PORTB &= ~(1 << PINB0);		
		juke(test_pulse1, test_triangle, test_noise, test_dpcm);
	}
}

//
// Main
//
int main(void)	
{
	initialize();
	
	/*
	while (1) ///////////// DEBUGGING ///////////////////
	{
		if ( s_debounce_time == 0 && s_pressed )
		{
			s_pressed = 0;

			// Check if button is still pressed
			if ( (PIND & (1 << PIND2)) == 0 )
			{
				juke_go = 1;
			}
		}
		
		if ( juke_go ) {
			phoenix();
		}
	}
	*/

	while(1)
	{
		// Read button press from user (up)
		if ( u_debounce_time == 0 && u_pressed )
		{
			u_pressed = 0;

			// Check if button is still pressed
			if ( (PIND & (1 << PIND3)) == 0 )
			{
				// If there is room to move
				if ( cursor_pos > 0 )
				{
					// Move cursor up
					cursor_pos = cursor_pos - 1;
					if ( cursor_pos < 2 ) {
						LCDGotoXY(0, cursor_pos);
					}
					else {
						LCDGotoXY(-4, cursor_pos);
					}
				}
			}
		} 	

		// Read button press from user (down)
		if ( d_debounce_time == 0 && d_pressed )
		{
			d_pressed = 0;

			// Check if button is still pressed
			if ( (PINB & (1 << PINB2)) == 0 )
			{
				// If there is room to move
				if ( cursor_pos < 3 )
				{
					// Move cursor down
					cursor_pos = cursor_pos + 1;
					if ( cursor_pos < 2 ) {
						LCDGotoXY(0, cursor_pos);
					}
					else {
						LCDGotoXY(-4, cursor_pos);
					}
				}
			}
		}	

		// Read button press from user (select)
		if ( s_debounce_time == 0 && s_pressed )
		{
			s_pressed = 0;

			// Check if button is still pressed
			if ( (PIND & (1 << PIND2)) == 0 )
			{
				// If there is room to move
				if ( cursor_pos == MODE )
				{
					// Raise flag to change mode
					change_mode = 1;
				}
				if ( cursor_pos == BRUISES )
				{
					// Raise flag to play bruises
					juke_go = BRUISES;
				}
				if ( cursor_pos == FOREVER )
				{
					// Raise flag to play faye
					juke_go = FOREVER;
				}
				if ( cursor_pos == PHOENIX ) {
					// Raise flag to play phoenix
					juke_go = PHOENIX;
				}
			}
		}
		
		// Carry out select actions; consume flags
		if ( change_mode )
		{
			// Consume flag
			change_mode = 0;

			// Change mode
			mode ^= 1;
			
			// Update LCD
			if ( mode == JUKE ) {
				CopyStringtoLCD(lcd_juke, 1, 0);
			}
			else {
				CopyStringtoLCD(lcd_affective, 1, 0);
			}

			// Reposition cursor
			if ( cursor_pos < 2 ) {
				LCDGotoXY(0, cursor_pos);
			}
			else {
				LCDGotoXY(-4, cursor_pos);
			}
		}
		else if ( juke_go )
		{
			if ( juke_go == BRUISES )
			{
				// Play bruises
				bruises();
			}
			else if ( juke_go == FOREVER )
			{
				// Play faye
				faye();
			}
			else if ( juke_go == PHOENIX )
			{
				// Play phoenix
				phoenix();
			}
		}


	} // end while

} // end main
